package com.dayou.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.configuration.BaseEntity;
import com.dayou.dao.DocumentMapper;
import com.dayou.dao.QuestionMapper;
import com.dayou.dto.DocumentDTO;
import com.dayou.entity.Configuration;
import com.dayou.entity.Document;
import com.dayou.entity.DocumentQuestion;
import com.dayou.entity.Question;
import com.dayou.service.IConfigurationService;
import com.dayou.service.IDocumentService;
import com.dayou.service.IQuestionService;
import com.dayou.utils.ConvertUtil;
import com.dayou.utils.TreeUtil;
import com.dayou.vo.DocumentCycleVO;
import com.dayou.vo.DocumentVO;
import com.dayou.vo.QuestionTreeVO;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.checkerframework.checker.units.qual.K;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/7/13
 * created with IntelliJ IDEA.
 */
@Service
public class DocumentServiceImpl extends ServiceImpl<DocumentMapper, Document> implements IDocumentService {

    @Autowired
    private DocumentMapper documentMapper;

    @Autowired
    private QuestionMapper questionMapper;

    @Autowired
    private IConfigurationService configurationService;

    @Override
    public DocumentCycleVO getDocumentCycleVO() {
        Configuration mini = configurationService.getOne(new LambdaQueryWrapper<Configuration>().eq(Configuration::getName, "mini").eq(BaseEntity::getDeleted, Boolean.FALSE));
        if ("false".equals(mini.getValue())){
            return new DocumentCycleVO();
        }
        DocumentCycleVO documentCycleVO = documentMapper.getDocumentCycleVO();
        List<List<Question>> results = new ArrayList<>();
        Long typeId = documentCycleVO.getTypeId();
        List<Question> children = Lists.newArrayList();
        children = findChildren(typeId, children);
        Map<Long,List<Question>> result = new LinkedHashMap<>();
        Map<Long, List<Question>> collect = children.stream().collect(Collectors.groupingBy(Question::getParentId));
        collect.entrySet().stream().sorted(Map.Entry.comparingByKey()).forEachOrdered(e->result.put(e.getKey(),e.getValue()));
        for(Map.Entry<Long,List<Question>> map :result.entrySet()){
            results.add(map.getValue());
        }
        documentCycleVO.setQuestions(results);
        return documentCycleVO;
    }

    @Override
    public IPage<DocumentVO> getPage(Page page, Document document) {
        IPage<DocumentVO> result = documentMapper.getPage(page,document);
        return result;
    }

    private List<Question> findChildren(Long rootId,List<Question> children){
        List<Question> list = questionMapper.findChildren(rootId);
        if (CollectionUtils.isNotEmpty(list)){
            children.addAll(list);
        }else {
            return children;
        }
        for (Question question : list){
            findChildren(question.getId(),children);
        }
        return children;
    }
}
